from dateutil.parser import parse
import gzip
from lxml import etree
from pmlab_lite.log import Event, EventLog
from . import constants as  c
from tqdm import tqdm
import re

NAMESPACES = {'xes': 'http://www.xes-standard.org/'}
ATTR_TYPES = ['string', 'date', 'boolean', 'int']

def import_from_xes(file):
    """
        Reads a *.xes-file and returns it as a log data structure. 

        Args:
            file: string literal path to a *.xes-file 

        Returns:
            log: the log represented as a data structure as in pmlab_lite.log
    """

    filename = __check_file_type(file)
    log = EventLog()
    tree = etree.parse(file)
    root = tree.getroot()
    ns = __check_namespace(root, 'xes')

    __import_extensions(log, root,ns)
    __import_globals(log, root,ns)
    __import_classifiers(log, root, ns)
    __import_log_attributes(log, root, ns)
    __import_traces(log, root, ns)

    return log

def __import_extensions(log: EventLog, root, ns: str):
    for ext in root.findall(ns+'extension', NAMESPACES):
        log.extensions[ext.attrib['name']] = {'prefix': ext.attrib['prefix'], 'uri': ext.attrib['uri']}

def __import_globals(log: EventLog, root, ns: str):
    globals = root.findall(ns+'global', NAMESPACES)
    for g in globals:
        scope = g.attrib['scope']
        log.globals[scope] = {}
        for attr_type in ATTR_TYPES:
            for attr in g.findall(ns+attr_type, NAMESPACES):
                key, value = __get_xes_key_value(attr, attr_type)
                log.globals[scope][key] = value

def __import_classifiers(log: EventLog, root, ns: str):
    classifiers = root.findall(ns+'classifier', NAMESPACES)

    if classifiers:
        for classifier in classifiers:
            name = classifier.attrib['name']
            attributes = classifier.attrib['keys']
            #resolve special case: wo 
            if "'" in attributes:
                attributes = re.findall(r'\'(.*?)\'', attributes)
            else:
                attributes = attributes.split()
            log.classifiers[name] = attributes 

def __import_log_attributes(log: EventLog, root, ns: str):
    for attr_type in ATTR_TYPES:
        for attr in root.findall(ns+attr_type, NAMESPACES):
            key, value = __get_xes_key_value(attr, attr_type)
            log.metadata[key] = value

def __import_traces(log: EventLog, root, ns: str):
    traces = root.findall(ns+'trace', NAMESPACES)

    for trace in tqdm(traces):
        
        # extract the case id
        if trace.find(ns+'string[@key="concept:name"]', NAMESPACES) is not None:
            case_id = trace.find(ns+'string[@key="concept:name"]', NAMESPACES).attrib['value']
        else:
            case_id = None
        log.add_trace(case_id)
        
        __import_trace_attributes(log, case_id, trace, ns)
        __import_events(log, case_id, trace, ns)

def __import_trace_attributes(log: EventLog, case_id, root, ns: str):
    for attr_type in ATTR_TYPES:
        for attr in root.findall(ns+attr_type, NAMESPACES):
            key, value = __get_xes_key_value(attr, attr_type)
            log.traces[case_id][key] = value

def __import_events(log: EventLog, case_id, root, ns):
    for e in root.findall(ns+'event', NAMESPACES):
            
        event = Event(case_id)

        for attr_type in ATTR_TYPES:
            for attr in e.findall(ns+attr_type, NAMESPACES):
                key, value = __get_xes_key_value(attr, attr_type)
                event[key] = value

        log.add_event(event)

def __get_xes_key_value(xes_attr, attr_type):
    """
        Parses XES Attributes depending on their respective data type.
        
        Returns the key, value as strings.
    """
    key = xes_attr.attrib['key']

    if attr_type == 'date':
        value = parse(xes_attr.attrib['value'])
    elif attr_type == 'int':
        value = int(xes_attr.attrib['value'])
    elif attr_type == 'boolean':
        value = xes_attr.attrib['value'].lower() == 'true'
    else:
        value = xes_attr.attrib['value']

    return key, value 

def __check_file_type(file) -> str:
    """
        Checks for possiblities of files to input in the importer. 
        
        Returns the needed filename as a string.
    """

    if isinstance(file, str):
        filename = file
        if filename.endswith('.gz'):
            file = gzip.open(filename, 'rb')
        else:
            pass  # Just send the filename to xmltree.parse
    else:
        filename = file.name

    return filename

def __check_namespace(root, namespace: str) -> str:
    """ 
        Checks for a special case of namespaces that occur in some XES files.
        They have NAMESPACES before their tags so "etree.find" can't normally find tags, e.g. bpi12 and bpi14 log.

        Returns the needed namespace tag as a String.
    """

    if (NAMESPACES[namespace] in root.tag):
        return 'xes:'
    else:
        return ''


def export_to_xes(log: EventLog, target_path: str):
    """Exprots an EventLog structure as stored by pmlab_lite to an *.xes-file.

    Args:
        log (EventLog): The Event Log to be exported.
        target_path (str): The path- and filename of the exported *.xes-file, e.g. "some_file.xes".
    """
    
    root = etree.Element('log')

    __export_extensions(log, root)
    __export_globals(log, root)
    __export_classifiers(log, root)
    __export_log_attributes(log, root)
    __export_traces(log, root)

    tree = etree.ElementTree(root)
    tree.write(target_path, pretty_print=True, xml_declaration=True, encoding="utf-8")

def __export_extensions(log: EventLog, root):
    for key in log.extensions:
        extension = etree.SubElement(root, 'extension')
        extension.set('name', key)
        extension.set('prefix', log.extensions[key]['prefix'])
        extension.set('uri', log.extensions[key]['uri'])

def __export_globals(log: EventLog, root):
    for scope in log.globals:
        g = etree.SubElement(root, 'global')
        g.set('scope', scope)
        __export_attributes(log.globals[scope], g)

def __export_classifiers(log: EventLog, root):
    for classi in log.classifiers.keys():
        classi_attributes = ['\'' + attr + '\'' if ' ' in attr else attr for attr in log.classifiers[classi]]
        classifier = etree.SubElement(root, 'classifier')
        
        classifier.set('name', classi)
        classifier.set('keys', " ".join(classi_attributes))

def __export_log_attributes(log: EventLog, root):
    __export_attributes(log.metadata, root)

def __export_traces(log: EventLog, root):
    for case_id in log.traces.keys():
        trace = etree.SubElement(root, 'trace')
        trace_attr = {key:log.traces[case_id][key] for key in log.traces[case_id] if key!='events'} #don't consider the events as a trace attribute, alternate solution: make all atributes a subdict
        __export_attributes(trace_attr, trace)
        __export_events_of_trace(log.traces[case_id]['events'], trace)

def __export_events_of_trace(trc: list, root):
    for evnt in trc:
        event = etree.SubElement(root, 'event')
        __export_attributes(evnt, event)

def __export_attributes(attributes: dict, root):
    for log_attr, log_attr_val in attributes.items():
        log_attr_type = type(log_attr_val).__name__
        xes_attr_type = __log_to_xes_attribute_type(log_attr_type)
        xes_attr_value = __get_xes_attribute_value(xes_attr_type, log_attr_val)

        attribute = etree.SubElement(root, xes_attr_type)
        attribute.set('key', log_attr)
        attribute.set('value', xes_attr_value)

def __log_to_xes_attribute_type(log_attr_type) -> str:
    if log_attr_type in c.xes_typenames:
        return c.xes_typenames[log_attr_type]
    else:
        return 'string'

def __get_xes_attribute_value(xes_attr_type, log_attr_val):
    if xes_attr_type == 'date':
        return log_attr_val.strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + log_attr_val.strftime('%z')[0:3] + ':' + log_attr_val.strftime('%z')[3:]
    elif xes_attr_type == 'boolean':
        return str(log_attr_val).lower()
    else:
        return str(log_attr_val)


from . import constants as c
from . import variables as v
import numpy as np


class Node():	
	#shows a marking node

	def __init__(self, marking_vector, predecessor, number):
		self.marking_vector = marking_vector	
		self.predecessor = predecessor
		self.cost_from_start = 0
		self.cost_to_end = 100000
		self.total_cost = self.cost_from_start + self.cost_to_end

		self.alignment = []			#it's of the form [ (t1,t1), (t2,>>), (>>,t3), ... ]
		self.remaining_trace = []
		self.active_transitions = [] 

		self.number = number

	#method for comparing two objects (python 3.7)
	def __lt__(self, other):
		return self.total_cost < other.total_cost
			
	def find_active_transitions(self, incidence_matrix):
		#looping over transitions of the synchronous product, to see which are active, given the marking of that node
		for i in range(0, incidence_matrix.shape[1]):										#im.shape[1] returns #columns = #transitions
			
			if np.all( (incidence_matrix[:, i] + self.marking_vector) >= 0 ):				#im[:,i] returns i-th column as list
				#transition i is active
				if i not in self.active_transitions:
					self.active_transitions.append(i)
	
	def starting_cost(self):
		self.cost_from_start = 1 * sum( [c.EPSILON if ((x[0]!=c.BLANK and x[1]!=c.BLANK) or ('tau' in x[0])) else 1 for x in self.alignment] )

	#calcucalte the remaining cost to the final marking,based on a heuristic
	def ending_cost(self, heuristic):
		self.cost_to_end = heuristic.heuristic_to_final(self)

	def update_costs(self, heuristic):
		self.starting_cost()
		self.ending_cost(heuristic)	
		self.total_cost = self.cost_from_start + self.cost_to_end

	def get_predecessor(self):
		return self.predecessor


import unittest
from pmlab_lite.pn import PetriNet, TraceNet, SynchronousProduct
from pmlab_lite.helper.viz import dot

import numpy as np
from scipy.optimize import minimize


edges = [(1,-1), (-1,2), (-1,3), (2,-2), (2,-5), (3,-2), (3,-6), (-2,4), (-2,5), (5,-5), (5,-6), (-5,1), (-6,1), (4,-3), (4,-4), (-3,2), (-4,3)]
#create example
net = PetriNet()
for i in range(1,6):
    net.add_place(i)
for c in '123456':
    net.add_transition(c)
for e in edges:
    net.add_edge(e[0], e[1])
net.add_marking(1)
dot.draw_petri_net(net)
net.get_marking()


def fit(X, params):
    return X.dot(params)

def cost_function(params, X, num_nonsync_moves: int):
    result = None
    for i in range(num_nonsync_moves):
        result += X[i]
    for i in range(num_nonsync_moves, len(X.shape[0])):
        result += X[i]
    return result


import unittest
from pmlab_lite.helper.io import xes
from pprint import pprint

a_names = ['FlagX1.xes', 'FlagX2.xes', 'LevelA1.xes', 'LevelA2.xes', 'LevelB1.xes', 'LevelB2.xes', 
           'LevelC1.xes', 'LevelC2.xes', 'LevelD1.xes', 'LevelD2.xes'] 

rl_names = ['BPIC12.xes', 'BPIC13_closed_problems.xes', 'BPIC13_incidents.xes', 'BPIC13_open_problems.xes', 
            'BPIC15_1.xes', 'BPIC15_2.xes', 'BPIC15_3.xes', 'BPIC15_4.xes', 'BPIC15_5.xes', 'BPIC17.xes', 'BPIC17_Offer_log.xes']

a_path = '../XES_Certification/Artificial/'
rl_path = '../XES_Certification/Real-life/'

a_log = a_path + a_names[1]
rl_log = rl_path + rl_names[0] 


#test pmlab-lite
log = xes.import_from_xes(a_log)

#test pm4py
from pm4py.objects.log.importer.xes import importer as xes_importer

test_log = xes_importer.apply(a_log, variant=xes_importer.Variants.ITERPARSE)
#test_log2 = xes_importer.apply(rl_log, variant=xes_importer.Variants.LINE_BY_LINE)
print()
pprint(test_log.classifiers)
print()
pprint(log.metadata)
print()
for i, key in enumerate(log.traces):
    print(key)
    pprint(log.traces[key])
    print()
    if i > 1:
        break
#xes.export_to_xes(log, "test.xes")


from pmlab_lite.pn import PetriNet
from . alignment import Alignment
from . node import Node
from . heuristic import Heuristic
from . import constants as c
from . import variables as v
import numpy as np
import heapq



class A_Star(Alignment):
	
	def __init__(self, synchronous_product, trace, heuristic: str='lp', alignments: int=1):
		Alignment.__init__(self)

		self.heuristic = Heuristic(heuristic)
		self.incidence_matrix     = v.incidence_matrix     = synchronous_product.incidence_matrix()
		self.transitions_by_index = v.transitions_by_index = synchronous_product.transitions_by_index()
		self.final_mark_vector    = v.final_mark_vector    = synchronous_product.get_final_marking()
		
		init_node = Node(synchronous_product.get_init_marking(), None, 0)
		init_node.remaining_trace = trace
		init_node.find_active_transitions(self.incidence_matrix)
		init_node.update_costs(self.heuristic)

		self.closed_list = []																	#initialise closed set
		self.open_list = [ (init_node.total_cost, init_node) ]									#initialise open set
		heapq.heapify(self.open_list)	

		
	def search(self):
		while len(self.open_list) > 0:
			heapq.heapify(self.open_list)
			current_node = heapq.heappop(self.open_list)[1]
			self.closed_list.append(current_node)

			if ( np.array_equal(current_node.marking_vector, self.final_mark_vector)):
				self.alignments.append(current_node)
				break
			else:
				self.__investigate(current_node)

		
		self.calc_results()
		#return self.alignments

	def calc_results(self):
		self._fitness()
		self._alignment_moves()
		self._model_moves()
		self._log_moves()

	#this funtion calls other functions to investigate the current node
	def __investigate(self, node):
		'''
		:param incidence_matrix: matrix of the synchronous product
		:param transitions by index: reverse of the dict 'net.transitions', to access transitions names by index
		'''

		#heuristic evaluation of active transitions
		for i in node.active_transitions:
			#make child node and update it's marking, i.e. the current marking after transition i was fired
			child_node = Node(self.incidence_matrix[:, i] + node.marking_vector, node, node.number+1)
			child_node.find_active_transitions(self.incidence_matrix)
			
			# --Synchronous move--
			if self.transitions_by_index[i].endswith("synchronous"):
				
				#update it's remaining trace
				child_node.remaining_trace = node.remaining_trace[1:]
				child_node.alignment = node.alignment + [ (self.transitions_by_index[i][:-12], self.transitions_by_index[i][:-12] ) ]
				
			# --Model       move--
			elif self.transitions_by_index[i].endswith("model"):
				
				#update it's remaining trace
				child_node.remaining_trace = node.remaining_trace[:]
				child_node.alignment = node.alignment + [ ( self.transitions_by_index[i][:-6], c.BLANK ) ]
					
			# --Log         move--
			elif self.transitions_by_index[i].endswith("log"):

				#update it's remaining trace
				child_node.remaining_trace = node.remaining_trace[1:]
				child_node.alignment = node.alignment + [ (c.BLANK, self.transitions_by_index[i][:-4] ) ]

			#update the child nodes costs
			child_node.update_costs(self.heuristic)
	
			#check whether it's in the closed list or it's a cheaper version of same marking
			self.__add_node(child_node)

	#deciding on whether or not to add a node to the open list
	def __add_node(self,node):
		
		#checking whether it is in the closed list
		ind = [k for k in range( len(self.closed_list) ) if np.array_equal(node.marking_vector, self.closed_list[k].marking_vector) ] #ind is a list like [12,34,10]
		if len(ind) > 0:
			pass
		
		#checking whether it is in the open list, update if we found it
		else:
			ind = [k for k in range(len(self.open_list)) if np.array_equal(node.marking_vector, self.open_list[k][1].marking_vector)]
			
			#at least once in open list
			if ind:
				for k in ind:
					if self.open_list[k][1].cost_from_start > node.cost_from_start:	
						self.open_list[k] = [node.total_cost, node]
					else: 
						continue
			#not in open list yet
			else:
				self.open_list.append([node.total_cost, node])


### xes constants

### holds the equivalent xes type names for the python type names
xes_typenames = {"str": 'string',
                 "datetime": 'date',
                 "Timestamp": 'date',
                 "int": 'int',
                 "float": 'float',
                 "bool": 'boolean',
                 "dict": 'list'
                }


from pmlab_lite.manipulable import Manipulable
from datetime import datetime

class Event(dict):
    """An event is the minimum observable unit of information. It actually is a dictionary with, at least, three attributes: 'case_id', 'activity_name' and 'timestamp'."""

    def __init__(self, case_id):
        self["case:id"] = case_id

    def get_activity_name(self):
        return self["concept:name"]

    def get_case_id(self):
        return self["case:id"]

    def get_timestamp(self):
        return self['time:timestamp']

    def get_attributes(self):
        return [attribute for attribute in self.keys()]


class EventCollection(Manipulable):
    """An event collection is an abstract iterable collection of events. This is the smallest collection of events."""

    def __iter__(self):
        return self

    def __next__(self) -> Event:
        raise NotImplemented


class EventLog(EventCollection):
    """
    An event log is a collection of events with a fixed size. It can also extend the 'Manipulable' (or whatever we call
    it) class. With this implementation, it is not possible to have trace attributes, but only events attributes.
    Direct access to single traces is also possible
    """

    def __init__(self):
        self.events = list() # events are still stored as list of events
        self.traces = dict() # traces have attributes and store events as under trace['events'] -> list of events
        self.A = set()
        self.classifiers = dict() # stores the classifiers, i.e. an identity for events
        self.globals = dict() # stores the globally defined attributes, i.e attributes every trace/event must contain
        self.extensions = dict() # stores log extensions
        self.metadata = dict() # stores log attributes

    def add_event(self, event: Event):
        self.events.append(event)
        case_id = event.get_case_id()
        self.traces[case_id] = self.traces.get(case_id, {})
        self.traces[case_id]['events'] = self.traces[case_id].get('events', []) + [event] 
        return self

    def add_trace(self, case_id):
        self.traces[case_id] = {}
        return self

    def activity_set(self):
        """Creates the set of unique activities for the log."""
        
        for event in self.events:
            self.A.add(event['concept:name'])

    def get_traces(self):
        """ Returns the list of events associated to the traces. """
        traces_only = []
        for key in self.traces:
            traces_only.append(self.traces[key]['events'])
        return traces_only

    def get_trace(self, case_id):
        return self.traces[case_id]

    def equal_by_classifier(self, event1: Event, event2: Event, classifier: str) -> bool:
        """ Return true if the given events are equal according to the classifier, false otherwise."""

        attrib1 = self.classifiers[classifier][0]
        attrib2 = self.classifiers[classifier][1]
        return event1[attrib1] == event2[attrib1] and event1[attrib2] == event2[attrib2]

    def filter_by_classifier(self, event, classifier: str) -> list:
        """Return a list of all events of the log that are equal to the given event by the given classifier. 

        Args:
            event (Event): [The Event to compare all the other events of the log to.]
            classifier (str): [A classifier, i.e. a list of attributes the events must have the same values for, to be considered equal.]
        """
        equal_events = [event]
        for event2 in self.events:
            if self.equal_by_classifier(event, event2, classifier) and event is not event2:
                equal_events.append(event2)
        
        return equal_events

    def __iter__(self):
        return self.events.__iter__()

    def __next__(self):
        return self.events.__iter__().__next__()

    def print_traces(self, start: int=0, num: int=1): #maybe change to print global attributes so printing is flexible to different logs
        """
            Prints specified number of traces from the log given the start index. 
            For instance to print 3 traces starting from the 4th trace: log.print_traces(4, 3)

            Args:
                start (int, optional): The index of the start trace to print. Defaults to 0.
                num (int, optional): The number of the traces to print from start. Defaults to 1.

            Raises:
                Index out of bounds Error: Num > len(log) - 1
        """  
        
        traces_in_range = list(self.traces.items())[start:start+num]               #gives us the traces as tuples: [ (case_id, [event1, event2, ...]), ... ] so we can index with numbers rather than case_id

        for i in range(len(traces_in_range)):
            print("TRACE_ID:",traces_in_range[i][0], "(trace number", i+start, ")")
            print("  {:28}{:31}{:15}".format("activity name", "time", "transition"))
            for event in traces_in_range[i][1]:
                activity = event.get_activity_name()
                time = event.get_timestamp().strftime("%m/%d/%Y %H:%M:%S.%f %Z")
                transition = event['lifecycle:transition']
                print("> {:28}{:31}{:15}".format(activity, time, transition)) #
            print()

    def print(self):
        """Prints the whole log."""
        
        self.print_traces(0, self.len)


from . node import Node
from . import constants as c
from . import variables as v
import numpy as np
from scipy.linalg import lstsq

class Heuristic():

	def __init__(self, variant):
		self.variant = variant

	def heuristic_to_final(self, node):
		if self.variant == "tl":
			return self.remaining_trace_length_heursitic(node)
		elif self.variant == "lp":
			return self.linear_programming_heursitic(node)
	
	def remaining_trace_length_heursitic(self, node):
		return len(node.remaining_trace) * c.EPSILON

	def linear_programming_heursitic(self, node):
		#Heuristic.heurisitic_to_final()
		b = np.array(v.final_mark_vector) - np.array(node.marking_vector) 
		x = np.linalg.lstsq(v.incidence_matrix, b, rcond=None)[0]
		#x2 = lstsq(v.incidence_matrix, b, cond=None)[0]
		# important note: with rcond=None, the the results in costs calculated my vary on different machines
		# on three machines tested the unrounded sum of each x was mostly equal, differences occured in +-10 digits after comma
		# after rounding up or down though, differences where bigger on each machine (e.g. machine1 rounds -0.000001 to 0
		# 			and machine2 rounds 0.00000001 to 1 -> making the sums differ in multiples of 1
		#with rcond=-1 the same results have been calculated on each machine
		#
		# also linalg.lstsq does not return the x fulfilling min c(x) 

		#rounding up or down, 
		x[x > 0] = 1
		x[x <= 0] = 0
		
		for key in v.transitions_by_index:
			if v.transitions_by_index[key].startswith('tau') or v.transitions_by_index[key].endswith('synchronous'):
				x[key] = 0
		
		return np.sum(x)


from . import constants as c 

#class to store te specifications if the found alignment(s)
class Alignment:
	def __init__(self):
		self.alignments = []
		self.alignment_moves = []
		self.model_moves = []
		self.log_moves = []
		self.fitness = []

	def _fitness(self):
		for node in self.alignments:
			u = node.alignment
			self.fitness.append( round ( len( [e for e in u if ((e[0]!=c.BLANK and e[1]!=c.BLANK) or ('tau' in e[0])) ]) / len(u), 3) )
		
	def _alignment_moves(self):
		for node in self.alignments:
			u = node.alignment
			self.alignment_moves.append( [e for e in u if ('tau' not in e[0])] )
		
	def _model_moves(self):
		for node in self.alignments:
			u = node.alignment
			self.model_moves.append( [e[0] for e in u if (e[0]!=c.BLANK and ('tau' not in e[0]))] )  
		
	def _log_moves(self):
		for node in self.alignments:
			u = node.alignment
			self.log_moves.append( [e[1] for e in u if e[1] != c.BLANK])

	def print_alignment(self):
		row_one  = '| log trace          |'
		separator = ''
		row_two  = '| execution sequence |'

		for node in self.alignments:
			for tup in node.alignment:
				row_one += ' ' + tup[0] + ' '*max(0, -(len(tup[0])-len(tup[1]))) + ' |'
				row_two += ' ' + tup[1] + ' '*max(0, -(len(tup[1])-len(tup[0]))) + ' |'

			print('-'*len(row_one))
			print(row_one)
			print('-'*len(row_one))
			print(row_two)
			print('-'*len(row_one))


import unittest
from pmlab_lite.pn import PetriNet, TraceNet, SynchronousProduct
from pmlab_lite.alignments import a_star
from pmlab_lite.helper.viz.dot import draw_synchronous_product, draw_a_star_search_space

#create running example
running_example = PetriNet()
transitions = ['As', 'Aa', 'Da1', 'Fa', 'Sso', 'Ro', 'Co', 'tau', 'Ao', 'Do', 'Aaa', 'Da2', 'Af']
edges = [(1,-1), (-1,2), (2,-3), (2,-2), (-3,11), (-2,3), (-2,5), (3,-4), (-4,4), (4,-8), (5,-5), (-5,6), (6,-6), (-6,7), (7,-7), (-7,5), (7,-8),
         (-8,8), (8,-9), (-9,9), (9,-11), (-11,11), (8,-10), (-10,10), (10,-12), (-12,11), (11,-13), (-13,12)]

for p in range(1,13):
  running_example.add_place(p)
for t in transitions:
  running_example.add_transition(t)
for e in edges:
  running_example.add_edge(e[0], e[1])

#first example
#create trace net
trace1 = ['As', 'Aa', 'Sso', 'Ro', 'Ao', 'Aaa', 'Aaa']
trace_net1 = TraceNet(trace1)

#create synchronous product
sp1 = SynchronousProduct(running_example, trace_net1)


a = a_star.A_Star(sp1, trace1, heuristic='lp')
a.search()
a.print_alignment()


#second example
#create trace net
trace2 = ['As', 'Aa', 'Fa', 'Aaa', 'Ao', 'Af']
trace_net2 = TraceNet(trace2)

#create synchronous product
sp2 = SynchronousProduct(running_example, trace_net2)


b = a_star.A_Star(sp2, trace2, heuristic='lp', alignments=2)
b.search()
print()
b.print_alignment()

#draw_a_star_search_space(b)
#draw_synchronous_product(sp2)